////////////////////////////////////////////////////////////////////////////////
// checkstyle: Checks Java source code for adherence to a set of rules.
// Copyright (C) 2001-2012  Oliver Burn
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package com.puppycrawl.tools.checkstyle.checks.annotation;

import com.puppycrawl.tools.checkstyle.BaseCheckTestSupport;
import com.puppycrawl.tools.checkstyle.DefaultConfiguration;
import java.io.File;
import org.junit.Test;

public class SuppressWarningsTest extends BaseCheckTestSupport
{
    /**
     * Tests SuppressWarnings with default regex.
     * @throws Exception
     */
    @Test
    public void testSingleDefault() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);

        final String[] expected = {
            "8:23:17: The warning '   ' cannot be suppressed at this location.",
            "11:27:17: The warning '' cannot be suppressed at this location.",
            "53:27:17: The warning '' cannot be suppressed at this location.",
            "64:47:17: The warning '' cannot be suppressed at this location.",
            "67:37:17: The warning '' cannot be suppressed at this location.",
            "72:46:17: The warning '   ' cannot be suppressed at this location.",
            "77:60:17: The warning '   ' cannot be suppressed at this location.",
            "82:93:17: The warning '' cannot be suppressed at this location.",
            "82:106:17: The warning '    ' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings all warnings disabled on everything.
     * @throws Exception
     */
    @Test
    public void testSingleAll() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", ".*");

        final String[] expected = {
            "5:19:17: The warning 'unchecked' cannot be suppressed at this location.",
            "8:23:17: The warning '   ' cannot be suppressed at this location.",
            "11:27:17: The warning '' cannot be suppressed at this location.",
            "17:23:17: The warning 'unused' cannot be suppressed at this location.",
            "20:27:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "25:31:17: The warning 'unused' cannot be suppressed at this location.",
            "29:35:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:23:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:23:17: The warning 'abcun' cannot be suppressed at this location.",
            "47:27:17: The warning 'unused' cannot be suppressed at this location.",
            "53:27:17: The warning '' cannot be suppressed at this location.",
            "56:27:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:33:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:47:17: The warning '' cannot be suppressed at this location.",
            "67:37:17: The warning '' cannot be suppressed at this location.",
            "67:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:46:17: The warning '   ' cannot be suppressed at this location.",
            "72:54:17: The warning 'unused' cannot be suppressed at this location.",
            "72:65:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:37:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:60:17: The warning '   ' cannot be suppressed at this location.",
            "77:68:17: The warning 'unused' cannot be suppressed at this location.",
            "82:47:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:93:17: The warning '' cannot be suppressed at this location.",
            "82:98:17: The warning 'foo' cannot be suppressed at this location.",
            "82:106:17: The warning '    ' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testSingleNoUnchecked() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*");

        final String[] expected = {
            "5:19:17: The warning 'unchecked' cannot be suppressed at this location.",
            "29:35:17: The warning 'unchecked' cannot be suppressed at this location.",
            "56:27:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:48:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:33:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:65:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:37:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:47:17: The warning 'unchecked' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked warning disabled on certain tokens.
     * @throws Exception
     */
    @Test
    public void testSingleNoUncheckedTokens() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*");
        checkConfig.addAttribute("tokens", "CLASS_DEF,METHOD_DEF");

        final String[] expected = {
            "5:19:17: The warning 'unchecked' cannot be suppressed at this location.",
            "29:35:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:33:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:65:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:37:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:47:17: The warning 'unchecked' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings un* warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testSingleNoUnWildcard() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", ".*un.*");

        final String[] expected = {
            "5:19:17: The warning 'unchecked' cannot be suppressed at this location.",
            "17:23:17: The warning 'unused' cannot be suppressed at this location.",
            "20:27:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "25:31:17: The warning 'unused' cannot be suppressed at this location.",
            "29:35:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:23:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:23:17: The warning 'abcun' cannot be suppressed at this location.",
            "47:27:17: The warning 'unused' cannot be suppressed at this location.",
            "56:27:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:48:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:33:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:54:17: The warning 'unused' cannot be suppressed at this location.",
            "72:65:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:37:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:68:17: The warning 'unused' cannot be suppressed at this location.",
            "82:47:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked, unused warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testSingleNoUncheckedUnused() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*|^unused$");

        final String[] expected = {
            "5:19:17: The warning 'unchecked' cannot be suppressed at this location.",
            "17:23:17: The warning 'unused' cannot be suppressed at this location.",
            "25:31:17: The warning 'unused' cannot be suppressed at this location.",
            "29:35:17: The warning 'unchecked' cannot be suppressed at this location.",
            "47:27:17: The warning 'unused' cannot be suppressed at this location.",
            "56:27:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:48:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:33:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:54:17: The warning 'unused' cannot be suppressed at this location.",
            "72:65:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:37:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:68:17: The warning 'unused' cannot be suppressed at this location.",
            "82:47:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings *, unchecked, unused warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testSingleNoUncheckedUnusedAll() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*|^unused$*|.*");

        final String[] expected = {
            "5:19:17: The warning 'unchecked' cannot be suppressed at this location.",
            "8:23:17: The warning '   ' cannot be suppressed at this location.",
            "11:27:17: The warning '' cannot be suppressed at this location.",
            "17:23:17: The warning 'unused' cannot be suppressed at this location.",
            "20:27:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "25:31:17: The warning 'unused' cannot be suppressed at this location.",
            "29:35:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:23:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:23:17: The warning 'abcun' cannot be suppressed at this location.",
            "47:27:17: The warning 'unused' cannot be suppressed at this location.",
            "53:27:17: The warning '' cannot be suppressed at this location.",
            "56:27:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:33:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:47:17: The warning '' cannot be suppressed at this location.",
            "67:37:17: The warning '' cannot be suppressed at this location.",
            "67:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:46:17: The warning '   ' cannot be suppressed at this location.",
            "72:54:17: The warning 'unused' cannot be suppressed at this location.",
            "72:65:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:37:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:60:17: The warning '   ' cannot be suppressed at this location.",
            "77:68:17: The warning 'unused' cannot be suppressed at this location.",
            "82:47:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:93:17: The warning '' cannot be suppressed at this location.",
            "82:98:17: The warning 'foo' cannot be suppressed at this location.",
            "82:106:17: The warning '    ' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsSingle.java"), expected);
    }

    /**
     * Tests SuppressWarnings with default regex.
     * @throws Exception
     */
    @Test
    public void testCompactDefault() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);

        final String[] expected = {
            "8:24:17: The warning '   ' cannot be suppressed at this location.",
            "11:41:17: The warning '' cannot be suppressed at this location.",
            "44:23:17: The warning '' cannot be suppressed at this location.",
            "53:27:17: The warning '' cannot be suppressed at this location.",

            "64:48:17: The warning '' cannot be suppressed at this location.",
            "64:76:17: The warning '' cannot be suppressed at this location.",

            "67:38:17: The warning '' cannot be suppressed at this location.",
            "72:47:17: The warning '   ' cannot be suppressed at this location.",
            "72:98:17: The warning '   ' cannot be suppressed at this location.",
            "77:61:17: The warning '   ' cannot be suppressed at this location.",
            "82:94:17: The warning '' cannot be suppressed at this location.",
            "82:107:17: The warning '   ' cannot be suppressed at this location.",
            "82:181:17: The warning '' cannot be suppressed at this location.",
            "82:194:17: The warning '   ' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);
    }

    /**
     * Tests SuppressWarnings all warnings disabled on everything.
     * @throws Exception
     */
    @Test
    public void testCompactAll() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", ".*");

        final String[] expected = {
            "5:20:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:33:17: The warning 'unused' cannot be suppressed at this location.",
            "8:24:17: The warning '   ' cannot be suppressed at this location.",
            "11:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "11:41:17: The warning '' cannot be suppressed at this location.",
            "17:24:17: The warning 'unused' cannot be suppressed at this location.",
            "20:28:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "20:42:17: The warning '    un' cannot be suppressed at this location.",
            "25:32:17: The warning 'unused' cannot be suppressed at this location.",
            "29:36:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:24:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:23:17: The warning '' cannot be suppressed at this location.",
            "47:28:17: The warning 'unused' cannot be suppressed at this location.",
            "47:38:17: The warning 'bleh' cannot be suppressed at this location.",
            "53:27:17: The warning '' cannot be suppressed at this location.",
            "56:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:49:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:48:17: The warning '' cannot be suppressed at this location.",
            "64:62:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:76:17: The warning '' cannot be suppressed at this location.",

            "67:38:17: The warning '' cannot be suppressed at this location.",
            "67:43:17: The warning 'unchecked' cannot be suppressed at this location.",

            "72:47:17: The warning '   ' cannot be suppressed at this location.",
            "72:55:17: The warning 'unused' cannot be suppressed at this location.",
            "72:66:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:98:17: The warning '   ' cannot be suppressed at this location.",
            "72:106:17: The warning 'unused' cannot be suppressed at this location.",
            "72:117:17: The warning 'unchecked' cannot be suppressed at this location.",

            "77:38:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:61:17: The warning '   ' cannot be suppressed at this location.",
            "77:69:17: The warning 'unused' cannot be suppressed at this location.",

            "82:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:94:17: The warning '' cannot be suppressed at this location.",
            "82:99:17: The warning 'foo' cannot be suppressed at this location.",
            "82:107:17: The warning '   ' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
            "82:135:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:181:17: The warning '' cannot be suppressed at this location.",
            "82:186:17: The warning 'foo' cannot be suppressed at this location.",
            "82:194:17: The warning '   ' cannot be suppressed at this location.",
            "82:202:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testCompactNoUnchecked() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*");

        final String[] expected = {
            "5:20:17: The warning 'unchecked' cannot be suppressed at this location.",
            "11:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "29:36:17: The warning 'unchecked' cannot be suppressed at this location.",
            "56:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:62:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:43:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:66:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:117:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:38:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:135:17: The warning 'unchecked' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);

    }

    /**
     * Tests SuppressWarnings unchecked warning disabled on certain tokens.
     * @throws Exception
     */
    @Test
    public void testCompactNoUncheckedTokens() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*");
        checkConfig.addAttribute("tokens", "CLASS_DEF");

        final String[] expected = {
            "5:20:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:62:17: The warning 'unchecked' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);
    }

    /**
     * Tests SuppressWarnings un* warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testCompactNoUnWildcard() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "un.*");

        final String[] expected = {
            "5:20:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:33:17: The warning 'unused' cannot be suppressed at this location.",
            "11:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "17:24:17: The warning 'unused' cannot be suppressed at this location.",
            "20:28:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "25:32:17: The warning 'unused' cannot be suppressed at this location.",
            "29:36:17: The warning 'unchecked' cannot be suppressed at this location.",
            "47:28:17: The warning 'unused' cannot be suppressed at this location.",
            "56:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:49:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:62:17: The warning 'unchecked' cannot be suppressed at this location.",

            "67:43:17: The warning 'unchecked' cannot be suppressed at this location.",

            "72:55:17: The warning 'unused' cannot be suppressed at this location.",
            "72:66:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:106:17: The warning 'unused' cannot be suppressed at this location.",
            "72:117:17: The warning 'unchecked' cannot be suppressed at this location.",

            "77:38:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:69:17: The warning 'unused' cannot be suppressed at this location.",

            "82:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
            "82:135:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:202:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked, unused warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testCompactNoUncheckedUnused() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*|^unused$");

        final String[] expected = {
            "5:20:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:33:17: The warning 'unused' cannot be suppressed at this location.",
            "11:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "17:24:17: The warning 'unused' cannot be suppressed at this location.",
            "25:32:17: The warning 'unused' cannot be suppressed at this location.",
            "29:36:17: The warning 'unchecked' cannot be suppressed at this location.",
            "47:28:17: The warning 'unused' cannot be suppressed at this location.",
            "56:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:62:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:43:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:55:17: The warning 'unused' cannot be suppressed at this location.",
            "72:66:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:106:17: The warning 'unused' cannot be suppressed at this location.",
            "72:117:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:38:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:69:17: The warning 'unused' cannot be suppressed at this location.",
            "82:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
            "82:135:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:202:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);
    }

    /**
     * Tests SuppressWarnings *, unchecked, unused warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testCompactNoUncheckedUnusedAll() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*|^unused$*|.*");

        final String[] expected = {
            "5:20:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:33:17: The warning 'unused' cannot be suppressed at this location.",
            "8:24:17: The warning '   ' cannot be suppressed at this location.",
            "11:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "11:41:17: The warning '' cannot be suppressed at this location.",
            "17:24:17: The warning 'unused' cannot be suppressed at this location.",
            "20:28:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "20:42:17: The warning '    un' cannot be suppressed at this location.",
            "25:32:17: The warning 'unused' cannot be suppressed at this location.",
            "29:36:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:24:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:23:17: The warning '' cannot be suppressed at this location.",
            "47:28:17: The warning 'unused' cannot be suppressed at this location.",
            "47:38:17: The warning 'bleh' cannot be suppressed at this location.",
            "53:27:17: The warning '' cannot be suppressed at this location.",
            "56:28:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:49:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:48:17: The warning '' cannot be suppressed at this location.",
            "64:62:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:76:17: The warning '' cannot be suppressed at this location.",

            "67:38:17: The warning '' cannot be suppressed at this location.",
            "67:43:17: The warning 'unchecked' cannot be suppressed at this location.",

            "72:47:17: The warning '   ' cannot be suppressed at this location.",
            "72:55:17: The warning 'unused' cannot be suppressed at this location.",
            "72:66:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:98:17: The warning '   ' cannot be suppressed at this location.",
            "72:106:17: The warning 'unused' cannot be suppressed at this location.",
            "72:117:17: The warning 'unchecked' cannot be suppressed at this location.",

            "77:38:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:61:17: The warning '   ' cannot be suppressed at this location.",
            "77:69:17: The warning 'unused' cannot be suppressed at this location.",

            "82:48:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:94:17: The warning '' cannot be suppressed at this location.",
            "82:99:17: The warning 'foo' cannot be suppressed at this location.",
            "82:107:17: The warning '   ' cannot be suppressed at this location.",
            "82:115:17: The warning 'unused' cannot be suppressed at this location.",
            "82:135:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:181:17: The warning '' cannot be suppressed at this location.",
            "82:186:17: The warning 'foo' cannot be suppressed at this location.",
            "82:194:17: The warning '   ' cannot be suppressed at this location.",
            "82:202:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsCompact.java"), expected);
    }

    /**
     * Tests SuppressWarnings with default regex.
     * @throws Exception
     */
    @Test
    public void testExpandedDefault() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);

        final String[] expected = {
            "8:30:17: The warning '   ' cannot be suppressed at this location.",
            "11:47:17: The warning '' cannot be suppressed at this location.",
            "44:29:17: The warning '' cannot be suppressed at this location.",
            "53:33:17: The warning '' cannot be suppressed at this location.",
            "64:54:17: The warning '' cannot be suppressed at this location.",
            "64:82:17: The warning '' cannot be suppressed at this location.",
            "67:44:17: The warning '' cannot be suppressed at this location.",
            "72:53:17: The warning '   ' cannot be suppressed at this location.",
            "72:104:17: The warning '   ' cannot be suppressed at this location.",
            "77:67:17: The warning '   ' cannot be suppressed at this location.",
            "82:100:17: The warning '' cannot be suppressed at this location.",
            "82:113:17: The warning '   ' cannot be suppressed at this location.",
            "82:187:17: The warning '' cannot be suppressed at this location.",
            "82:200:17: The warning '   ' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }

    /**
     * Tests SuppressWarnings all warnings disabled on everything.
     * @throws Exception
     */
    @Test
    public void testExpandedAll() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", ".*");

        final String[] expected = {
            "5:26:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:39:17: The warning 'unused' cannot be suppressed at this location.",
            "8:30:17: The warning '   ' cannot be suppressed at this location.",
            "11:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "11:47:17: The warning '' cannot be suppressed at this location.",
            "17:30:17: The warning 'unused' cannot be suppressed at this location.",
            "20:34:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "20:48:17: The warning '    un' cannot be suppressed at this location.",
            "25:38:17: The warning 'unused' cannot be suppressed at this location.",
            "29:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:30:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:29:17: The warning '' cannot be suppressed at this location.",
            "47:34:17: The warning 'unused' cannot be suppressed at this location.",
            "47:44:17: The warning 'bleh' cannot be suppressed at this location.",
            "53:33:17: The warning '' cannot be suppressed at this location.",
            "56:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:55:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:40:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:54:17: The warning '' cannot be suppressed at this location.",
            "64:68:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:82:17: The warning '' cannot be suppressed at this location.",
            "67:44:17: The warning '' cannot be suppressed at this location.",
            "67:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:53:17: The warning '   ' cannot be suppressed at this location.",
            "72:61:17: The warning 'unused' cannot be suppressed at this location.",
            "72:72:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:104:17: The warning '   ' cannot be suppressed at this location.",
            "72:112:17: The warning 'unused' cannot be suppressed at this location.",
            "72:123:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:44:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:67:17: The warning '   ' cannot be suppressed at this location.",
            "77:75:17: The warning 'unused' cannot be suppressed at this location.",
            "82:54:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:100:17: The warning '' cannot be suppressed at this location.",
            "82:105:17: The warning 'foo' cannot be suppressed at this location.",
            "82:113:17: The warning '   ' cannot be suppressed at this location.",
            "82:121:17: The warning 'unused' cannot be suppressed at this location.",
            "82:141:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:187:17: The warning '' cannot be suppressed at this location.",
            "82:192:17: The warning 'foo' cannot be suppressed at this location.",
            "82:200:17: The warning '   ' cannot be suppressed at this location.",
            "82:208:17: The warning 'unused' cannot be suppressed at this location.",

        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testExpandedNoUnchecked() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*");

        final String[] expected = {
            "5:26:17: The warning 'unchecked' cannot be suppressed at this location.",
            "11:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "29:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "56:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:55:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:40:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:68:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:72:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:123:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:44:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:54:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:141:17: The warning 'unchecked' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }

    /**
     * Tests SuppressWarnings unchecked warning disabled on certain tokens.
     * @throws Exception
     */
    @Test
    public void testExpandedNoUncheckedTokens() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*");
        checkConfig.addAttribute("tokens", "CLASS_DEF");

        final String[] expected = {
            "5:26:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:40:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:68:17: The warning 'unchecked' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }

    /**
     * Tests SuppressWarnings un* warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testExpandedNoUnWildcard() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "un.*");

        final String[] expected = {
            "5:26:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:39:17: The warning 'unused' cannot be suppressed at this location.",
            "11:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "17:30:17: The warning 'unused' cannot be suppressed at this location.",
            "20:34:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "25:38:17: The warning 'unused' cannot be suppressed at this location.",
            "29:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "47:34:17: The warning 'unused' cannot be suppressed at this location.",
            "56:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:55:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:40:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:68:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:61:17: The warning 'unused' cannot be suppressed at this location.",
            "72:72:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:112:17: The warning 'unused' cannot be suppressed at this location.",
            "72:123:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:44:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:75:17: The warning 'unused' cannot be suppressed at this location.",
            "82:54:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:121:17: The warning 'unused' cannot be suppressed at this location.",
            "82:141:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:208:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }


    /**
     * Tests SuppressWarnings unchecked, unused warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testExpandedNoUncheckedUnused() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*|^unused$");

        final String[] expected = {
            "5:26:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:39:17: The warning 'unused' cannot be suppressed at this location.",
            "11:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "17:30:17: The warning 'unused' cannot be suppressed at this location.",
            "25:38:17: The warning 'unused' cannot be suppressed at this location.",
            "29:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "47:34:17: The warning 'unused' cannot be suppressed at this location.",
            "56:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:55:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:40:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:68:17: The warning 'unchecked' cannot be suppressed at this location.",
            "67:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:61:17: The warning 'unused' cannot be suppressed at this location.",
            "72:72:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:112:17: The warning 'unused' cannot be suppressed at this location.",
            "72:123:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:44:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:75:17: The warning 'unused' cannot be suppressed at this location.",
            "82:54:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:121:17: The warning 'unused' cannot be suppressed at this location.",
            "82:141:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:208:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }

    /**
     * Tests SuppressWarnings *, unchecked, unused warning disabled on everything.
     * @throws Exception
     */
    @Test
    public void testExpandedNoUncheckedUnusedAll() throws Exception
    {
        DefaultConfiguration checkConfig = createCheckConfig(SuppressWarningsCheck.class);
        checkConfig.addAttribute("format", "^unchecked$*|^unused$*|.*");

        final String[] expected = {
            "5:26:17: The warning 'unchecked' cannot be suppressed at this location.",
            "5:39:17: The warning 'unused' cannot be suppressed at this location.",
            "8:30:17: The warning '   ' cannot be suppressed at this location.",
            "11:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "11:47:17: The warning '' cannot be suppressed at this location.",
            "17:30:17: The warning 'unused' cannot be suppressed at this location.",
            "20:34:17: The warning 'unforgiven' cannot be suppressed at this location.",
            "20:48:17: The warning '    un' cannot be suppressed at this location.",
            "25:38:17: The warning 'unused' cannot be suppressed at this location.",
            "29:42:17: The warning 'unchecked' cannot be suppressed at this location.",
            "37:30:17: The warning 'abcun' cannot be suppressed at this location.",
            "44:29:17: The warning '' cannot be suppressed at this location.",
            "47:34:17: The warning 'unused' cannot be suppressed at this location.",
            "47:44:17: The warning 'bleh' cannot be suppressed at this location.",
            "53:33:17: The warning '' cannot be suppressed at this location.",
            "56:34:17: The warning 'unchecked' cannot be suppressed at this location.",
            "59:55:17: The warning 'unchecked' cannot be suppressed at this location.",

            "64:40:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:54:17: The warning '' cannot be suppressed at this location.",
            "64:68:17: The warning 'unchecked' cannot be suppressed at this location.",
            "64:82:17: The warning '' cannot be suppressed at this location.",
            "67:44:17: The warning '' cannot be suppressed at this location.",
            "67:49:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:53:17: The warning '   ' cannot be suppressed at this location.",
            "72:61:17: The warning 'unused' cannot be suppressed at this location.",
            "72:72:17: The warning 'unchecked' cannot be suppressed at this location.",
            "72:104:17: The warning '   ' cannot be suppressed at this location.",
            "72:112:17: The warning 'unused' cannot be suppressed at this location.",
            "72:123:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:44:17: The warning 'unchecked' cannot be suppressed at this location.",
            "77:67:17: The warning '   ' cannot be suppressed at this location.",
            "77:75:17: The warning 'unused' cannot be suppressed at this location.",
            "82:54:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:100:17: The warning '' cannot be suppressed at this location.",
            "82:105:17: The warning 'foo' cannot be suppressed at this location.",
            "82:113:17: The warning '   ' cannot be suppressed at this location.",
            "82:121:17: The warning 'unused' cannot be suppressed at this location.",
            "82:141:17: The warning 'unchecked' cannot be suppressed at this location.",
            "82:187:17: The warning '' cannot be suppressed at this location.",
            "82:192:17: The warning 'foo' cannot be suppressed at this location.",
            "82:200:17: The warning '   ' cannot be suppressed at this location.",
            "82:208:17: The warning 'unused' cannot be suppressed at this location.",
        };

        verify(checkConfig, getPath("annotation" + File.separator + "SuppressWarningsExpanded.java"), expected);
    }
}
